#ifndef __DALI_WIDGET_VIEW_INTERNAL_WIDGET_VIEW_H__
#define __DALI_WIDGET_VIEW_INTERNAL_WIDGET_VIEW_H__

/*
 * Samsung API
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// INTERNAL INCLUDES
#include <public_api/widget_view/widget_view.h>

// EXTERNAL INCLUDES
#include <dali/public-api/adaptor-framework/native-image-source.h>
#include <dali-toolkit/public-api/controls/control-impl.h>
#include <dali-toolkit/public-api/controls/text-controls/text-label.h>
#include <dali-toolkit/public-api/controls/image-view/image-view.h>
#include <wayland-extension/tizen-remote-surface-client-protocol.h>
#include <tbm_surface.h>
#include <bundle.h>

namespace Dali
{

namespace WidgetView
{

namespace Internal
{

class WidgetView : public Toolkit::Internal::Control
{
public:

  /**
   * @copydoc Dali::WidgetView::WidgetView::New
   */
  static Dali::WidgetView::WidgetView New( const std::string& widgetId, const std::string& contentInfo, int width, int height, float updatePeriod );

  /**
   * @copydoc Dali::WidgetView::WidgetView::PauseWidget
   */
  bool PauseWidget();

  /**
   * @copydoc Dali::WidgetView::WidgetView::ResumeWidget
   */
  bool ResumeWidget();

  /**
   * @copydoc Dali::WidgetView::WidgetView::CancelTouchEvent
   */
  bool CancelTouchEvent();

  /**
   * @copydoc Dali::WidgetView::WidgetView::ActivateFaultedWidget
   */
  void ActivateFaultedWidget();

  void SendWidgetEvent( int event );

public: // Internal API

  /**
   * @brief Called when a property of an object of this type is set.
   * @SINCE_1_1.38
   * @param[in] object The object whose property is set.
   * @param[in] index The property index.
   * @param[in] value The new property value.
   */
  static void SetProperty( BaseObject* object, Property::Index index, const Property::Value& value );

  /**
   * @brief Called to retrieve a property of an object of this type.
   * @SINCE_1_1.38
   * @param[in] object The object whose property is to be retrieved.
   * @param[in] index The property index.
   * @return The current value of the property.
   */
  static Property::Value GetProperty( BaseObject* object, Property::Index propertyIndex );

  /**
   * @brief Performs actions as requested using the action name.
   * @SINCE_1_1.38
   * @param[in] object The object on which to perform the action.
   * @param[in] actionName The action to perform.
   * @param[in] attributes The attributes with which to perfrom this action.
   * @return True if action has been accepted by this control
   */
  static bool DoAction( BaseObject* object, const std::string& actionName, const Property::Map& attributes );

  static bool DoConnectSignal( BaseObject* object, ConnectionTrackerInterface* tracker, const std::string& signalName, FunctorDelegate* functor );

  const std::string& GetWidgetId() const;

  const std::string& GetInstanceId() const;

  const std::string& GetContentInfo();

  const std::string& GetTitle();

  float GetUpdatePeriod() const;

  void ShowPreview( bool show );

  bool IsPreviewVisible();

  void ShowStateText( bool show );

  bool IsStateTextVisible();

  bool IsWidgetFaulted();

  void SetPermanentDelete( bool permanentDelete );

// ToDo: This funstion will be used for widgetDeletedSignal
  void RemoveObjectView();

  bool IsPermanentDelete();

  bool IsWidgetImageView();

  void UpdateImageSource( tbm_surface_h source );

  void CreateWidgetImageView();

  void ConnectSignal( tizen_remote_surface* surface );

  bool OnTouch( Dali::Actor actor, const Dali::TouchData& event );

  bool OnWheelEvent( Dali::Actor actor, const Dali::WheelEvent& event );

public: //Signals

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetAddedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetAddedSignal();

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetDeletedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetDeletedSignal();

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetCreationAbortedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetCreationAbortedSignal();

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetContentUpdatedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetContentUpdatedSignal();

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetUpdatePeriodChangedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetUpdatePeriodChangedSignal();

  /**
   * @copydoc Dali::WidgetView::WidgetView::WidgetFaultedSignal
   */
  Dali::WidgetView::WidgetView::WidgetViewSignalType& WidgetFaultedSignal();

protected:

  /**
   * Construct a new WidgetView.
   */
  WidgetView();

  /**
   * Construct a new WidgetView.
   */
  WidgetView( const std::string& widgetId, const std::string& contentInfo, int width, int height, float updatePeriod );

  /**
   * A reference counted object may only be deleted by calling Unreference()
   */
  virtual ~WidgetView();

private: // From Control

  /**
   * @copydoc Toolkit::Control::OnInitialize()
   */
  virtual void OnInitialize();

  /**
   * @copydoc Toolkit::Control::OnStageConnect()
   */
  virtual void OnStageConnection( int depth );

  /**
   * @copydoc Toolkit::Control::OnStageDisconnection()
   */
  virtual void OnStageDisconnection();

  /**
   * @copydoc Toolkit::Control::OnSizeSet()
   */
  virtual void OnSizeSet( const Vector3& targetSize );

  /**
   * @copydoc Toolkit::Control::GetNaturalSize
   */
  virtual Vector3 GetNaturalSize();

  /**
   * @copydoc Toolkit::Control::GetHeightForWidth()
   */
  virtual float GetHeightForWidth( float width );

  /**
   * @copydoc Toolkit::Control::GetWidthForHeight()
   */
  virtual float GetWidthForHeight( float height );

private:

  // Undefined
  WidgetView( const WidgetView& );

  // Undefined
  WidgetView& operator= ( const WidgetView& );

private:

  Toolkit::ImageView mWidgetImageView;     ///< Widget content
  Toolkit::ImageView mPreviewImage;   ///< Preview image
  Toolkit::TextLabel mStateText;      ///< State text
  Dali::NativeImageSourcePtr mImageSource;

  std::string mWidgetId;
  std::string mInstanceId;
  std::string mContentInfo;
  std::string mTitle;

  bundle* mBundle;

  int mWidth;
  int mHeight;
  int mPid;
  float mUpdatePeriod;

  bool mPreviewVisible;
  bool mStateTextVisible;
  bool mPermanentDelete;

  tizen_remote_surface* mRemoteSurface;

  // Signals
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetAddedSignal;
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetDeletedSignal;
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetCreationAbortedSignal;
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetContentUpdatedSignal;
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetUpdatePeriodChangedSignal;
  Dali::WidgetView::WidgetView::WidgetViewSignalType mWidgetFaultedSignal;
};

} // namespace Internal

// Helpers for public-api forwarding methods

inline Internal::WidgetView& GetImplementation( WidgetView& widgetView )
{
  DALI_ASSERT_ALWAYS( widgetView );

  Dali::RefObject& handle = widgetView.GetImplementation();

  return static_cast<Internal::WidgetView&>( handle );
}

inline const Internal::WidgetView& GetImplementation( const WidgetView& widgetView )
{
  DALI_ASSERT_ALWAYS( widgetView );

  const Dali::RefObject& handle = widgetView.GetImplementation();

  return static_cast<const Internal::WidgetView&>( handle );
}

} // namespace WidgetView

} // namespace Dali

#endif // __DALI_WIDGET_VIEW_INTERNAL_WIDGET_VIEW_H__
